@@ -6,4 +6,11 @@
 # license.
 #######################################################################
 
-from steve._version import __version__, __releasedate__
+# See http://www.python.org/dev/peps/pep-0386/
+# Examples:
+# * 1.0.dev
+# * 1.0a2
+# * 1.0b2
+# * 1.0
+__version__ = '0.2'
+__releasedate__ = '20130317'
