@@ -15,24 +15,38 @@ import sys
 import textwrap
 from functools import wraps
 
+import html2text
 import vidscraper
 
+
 YOUTUBE_EMBED = {
     'object': (
         '<object width="640" height="390"><param name="movie" '
-        'value="http://youtube.com/v/%(guid)s?version=3&amp;hl=en_US"></param>'
+        'value="http://youtube.com/v/{guid}?version=3&amp;hl=en_US"></param>'
         '<param name="allowFullScreen" value="true"></param>'
         '<param name="allowscriptaccess" value="always"></param>'
-        '<embed src="http://youtube.com/v/%(guid)s?version=3&amp;hl=en_US" '
+        '<embed src="http://youtube.com/v/{guid}?version=3&amp;hl=en_US" '
         'type="application/x-shockwave-flash" width="640" '
         'height="390" allowscriptaccess="always" '
         'allowfullscreen="true"></embed></object>'),
     'iframe': (
         '<iframe id="player" width="640" height="390" frameborder="0" '
-        'src="http://youtube.com/v/%(guid)s"></iframe>')
+        'src="http://youtube.com/v/{guid}"></iframe>')
     }
 
 
+class SteveException(Exception):
+    """Base steve exception"""
+    def __init__(self, *args, **kwargs):
+        self.__dict__.update(kwargs)
+        super(SteveException, self).__init__(*args)
+
+
+class ConfigNotFound(SteveException):
+    """Denotes the config file couldn't be found"""
+    pass
+
+
 class BetterArgumentParser(argparse.ArgumentParser):
     def __init__(self, *args, **kw):
         if 'byline' in kw:
@@ -52,10 +66,6 @@ class BetterArgumentParser(argparse.ArgumentParser):
         argparse.ArgumentParser.print_usage(self, file)
 
 
-class ConfigNotFound(Exception):
-    """Denotes the config file couldn't be found"""
-
-
 def with_config(fun):
     """Decorator that passes config as first argument
 
@@ -70,7 +80,7 @@ def with_config(fun):
 
     >>> @with_config
     ... def config_printer(cfg):
-    ...     print 'Config!: %r' % cfg
+    ...     print 'Config!: {0!r}'.format(cfg)
     ...
     >>> config_printer()  # if it found a config
     Config! ...
@@ -144,14 +154,15 @@ def get_project_config():
 
 
 def get_from_config(cfg, key, section='project',
-                    error='"%(key)s" must be defined in steve.ini file.'):
+                    error='"{key}" must be defined in steve.ini file.'):
     """Retrieves specified key from config or errors
 
     :arg cfg: the configuration
     :arg key: key to retrieve
     :arg section: the section to retrieve the key from
     :arg error: the error to print to stderr if the key is not
-        there or if the value is empty
+        there or if the value is empty. ``{key}`` gets filled in
+        with the key
 
     """
     try:
@@ -161,7 +172,7 @@ def get_from_config(cfg, key, section='project',
     except ConfigParser.NoOptionError:
         pass
 
-    err(error % {'key': key})
+    err(error.format(key=key))
     return None
 
 
@@ -260,17 +271,18 @@ def vidscraper_to_dict(video, youtube_embed=None):
                 item['video_flv_url'] = f.url
                 item['video_flv_download_only'] = False
             else:
-                raise ValueError('No clue what to do with %s' % f.mime_type)
+                raise ValueError(
+                    'No clue what to do with {0}'.format(f.mime_type))
 
     item['embed'] = video.embed_code
 
     if (youtube_embed is not None and 'youtube.com' in video.link
-        and hasattr(video, 'guid')):
+            and hasattr(video, 'guid')):
 
         guid = video.guid
         if guid:
             guid = video.guid.split('/')[-1]
-            item['embed'] = youtube_embed % {'guid': guid}
+            item['embed'] = youtube_embed.format(guid=guid)
 
     return item
 
@@ -284,15 +296,13 @@ def get_video_requirements():
 
 
 def _required(data):
-    if (data['null']
-        or data['has_default']
-        or data['empty_strings']):
+    if (data['null'] or data['has_default'] or data['empty_strings']):
         return False
     return True
 
 
-def verify_json(data, category=None):
-    """Verify the data in a single json file.
+def verify_video_data(data, category=None):
+    """Verify the data in a single json file for a video.
 
     :param data: The parsed contents of a JSON file. This should be a
         Python dict.
@@ -305,8 +315,8 @@ def verify_json(data, category=None):
         data file has to have a category.
 
     :returns: list of error strings.
-    """
 
+    """
     # TODO: rewrite this to return a dict of fieldname -> list of
     # errors
     errors = []
@@ -320,40 +330,53 @@ def verify_json(data, category=None):
             # Category is a special case since we can specify it
             # in the steve.ini file.
 
-            if category is None and key not in data:
+            if not category and key not in data:
                 errors.append(
                     '"category" must be in either steve.ini or data file')
-            elif key in data and data[key] != category:
+            elif (key in data and (
+                    category is not None and data[key] != category)):
                 errors.append(
-                    '"%s" field does not match steve.ini category' % key)
+                    '"{0}" field does not match steve.ini category'.format(
+                        key))
 
-        elif _required(req) and key not in data:
-            errors.append('"%s" field is required' % key)
+        elif key not in data:
+            # Required data must be there.
 
-        elif req['null'] and data.get(key) == None:
-            continue
+            # TODO: We add title here because this is the client side
+            # of the API and that's a special case that's differen
+            # than the data model which is where the video_reqs.json
+            # are derived. That should get fixed in richard.
+            if _required(req) or key == 'title':
+                errors.append('"{0}" field is required'.format(key))
 
         elif req['type'] == 'IntegerField':
             if not isinstance(data[key], int):
-                errors.append('"%s" field must be an int' % key)
+                if (_required(req)
+                    or (not _required(req) and data[key] is not None)):
+
+                    errors.append('"{0}" field must be an int'.format(key))
             elif req['choices'] and data[key] not in req['choices']:
-                errors.append('"%s" field must be one of %s' % (
-                        key, req.choices))
+                errors.append('"{0}" field must be one of {1}'.format(
+                        key, req['choices']))
 
         elif req['type'] == 'TextField':
             if not req['empty_strings'] and not data[key]:
-                errors.append('"%s" field can\'t be an empty string' % key)
+                errors.append(
+                    '"{0}" field can\'t be an empty string'.format(key))
             elif not data[key]:
                 continue
-            elif req['html'] and not '<' in data[key]:
-                errors.append('"%s" field should be HTML formatted' % key)
 
         elif req['type'] == 'TextArrayField':
             for mem in data[key]:
                 if not mem:
-                    errors.append('"%s" field has empty strings in it' % key)
+                    errors.append(
+                        '"{0}" field has empty strings in it'.format(key))
                     break
 
+        elif req['type'] == 'BooleanField':
+            if data[key] not in (True, False):
+                errors.append('"{0}" field has non-boolean value'.format(key))
+
     required_keys = [req['name'] for req in requirements]
 
     # Second check to make sure there aren't fields that shouldn't
@@ -365,7 +388,7 @@ def verify_json(data, category=None):
             continue
 
         if key not in required_keys:
-            errors.append('"%s" field shouldn\'t be there.' % key)
+            errors.append('"{0}" field shouldn\'t be there.'.format(key))
 
     return errors
 
@@ -376,7 +399,7 @@ def verify_json_files(json_files, category=None):
     Prints the output
 
     :param json_files: list of (filename, parsed json data) tuples to
-        call verify_json on
+        call :py:func:`verify_video_data` on
 
     :param category: The category as specified in the steve.ini file.
 
@@ -391,7 +414,7 @@ def verify_json_files(json_files, category=None):
     filename_to_errors = {}
 
     for filename, data in json_files:
-        filename_to_errors[filename] = verify_json(data, category)
+        filename_to_errors[filename] = verify_video_data(data, category)
 
     return filename_to_errors
 
@@ -415,7 +438,7 @@ def err(*output, **kw):
 
     """
     output = 'Error: ' + ' '.join([str(o) for o in output])
-    if kw.get('wrap') != False:
+    if kw.get('wrap') is not False:
         output = '\n'.join(wrap(output, kw.get('indent', '')))
     elif kw.get('indent'):
         indent = kw['indent']
@@ -437,7 +460,7 @@ def out(*output, **kw):
 
     """
     output = ' '.join([stringify(o) for o in output])
-    if kw.get('wrap') != False:
+    if kw.get('wrap') is not False:
         output = '\n'.join(wrap(output, kw.get('indent', '')))
     elif kw.get('indent'):
         indent = kw['indent']
@@ -472,7 +495,7 @@ def load_json_files(config):
             data.append((fn, json.load(fp)))
             fp.close()
         except Exception, e:
-            err('Problem with %s' % fn, wrap=False)
+            err('Problem with {0}'.format(fn), wrap=False)
             raise e
 
     return data
@@ -558,9 +581,23 @@ def scrapevideo(video_url):
     data['url'] = video_url
     if 'youtube.com' in video_url and 'guid' in data and data['guid']:
         guid = data['guid'].split('/')[-1]
-        data['object_embed_code'] = (YOUTUBE_EMBED['object'] %
-                                     {'guid': guid})
-        data['iframe_embed_code'] = (YOUTUBE_EMBED['iframe'] %
-                                     {'guid': guid})
+        data['object_embed_code'] = YOUTUBE_EMBED['object'].format(guid=guid)
+        data['iframe_embed_code'] = YOUTUBE_EMBED['iframe'].format(guid=guid)
 
     return data
+
+
+def html_to_markdown(text):
+    """Converts an HTML string to equivalent Markdown
+
+    :arg text: the HTML string to convert
+
+    :returns: Markdown string
+
+    Example:
+
+    >>> html_to_markdown('<p>this is <b>html</b>!</p>')
+    u'this is **html**!'
+
+    """
+    return html2text.html2text(text).strip()
